@@ -1,18 +1,19 @@
-#!/usr/bin/perl
-
 # -----------------------------------------------------------------------------
 #
 #  Nes by Skriptke
-#  Copyright 2009 - 2010 Enrique F. Castañón
+#  Copyright 2009 - 2010 Enrique F. Castañón Barbero
 #  Licensed under the GNU GPL.
 #
+#  CPAN:
+#  http://search.cpan.org/dist/Nes/
+#
 #  Sample:
 #  http://nes.sourceforge.net/
 #
 #  Repository:
 #  http://github.com/Skriptke/nes
 # 
-#  Version 1.00_03
+#  Version 1.04
 #
 #  Singleton.pm
 #
@@ -21,22 +22,20 @@
 {
   package Nes::Singleton;
   
-  my $singleton;
+  my $instance;
 
   sub new {
     my $class = shift;
-    my $self  = $singleton || bless {}, $class;
-    my ( $file ) = @_;
-
-    utl::cleanup(\$singleton) if $ENV{'MOD_PERL'};   
-
-    if ( $singleton ) {
+    my $self  = $instance || bless {}, $class;
+    my ( $file, $nes_top_dir, $nes_dir ) = @_;
+    
+    if ( $instance ) {
       $self->{'container'} = nes_container->get_obj();
       $self->{'this_template_name'} = $self->{'container'}->{'file_name'};
       $self->{'top_template_name'}  = $self->{'top_container'}->{'file'};      
       return $self;
     } else {
-      $singleton = $self;
+      $instance = $self;
     }
 
     $self->{'file'} = $ENV{'PATH_TRANSLATED'} || $file;
@@ -49,12 +48,13 @@
   
     die "No template defined: $@" if !$self->{'file'};
 
-    $self->{'CFG'}           = Nes::Setting->new();
+    $self->{'CFG'}           = Nes::Setting->new( $nes_top_dir, $nes_dir  );
     $self->{'top_container'} = nes_top_container->new( $self->{'file'}, $dir );
     $self->{'container'}     = nes_container->get_obj();
     $self->{'cookies'}       = nes_cookie->get_obj();
     $self->{'session'}       = nes_session->get_obj();
-    $self->{'query'}         = nes_query->get_obj();      
+    $self->{'query'}         = nes_query->get_obj();
+    $self->{'register'}      = nes_register->get_obj();
     $self->{'nes'}           = $self->{'top_container'}->{'nes'};
     $self->{'this_template_name'} = $self->{'container'}->{'file_name'};
     $self->{'top_template_name'}  = $self->{'top_container'}->{'file'};
@@ -79,7 +79,6 @@
     $self->{'container'}->go(); 
     $self->{'top_container'}->{'container'}->out();
     $self->{'container'}->forget();
-    $self->{'top_container'}->forget();
 
     return;
   }
@@ -94,6 +93,30 @@
 
     return;
   }
+  
+  sub add {
+    my $self = shift;
+    my %tags;
+    (%tags) = @_;
+
+    $self->{'container'}->add_tags(%tags);
+
+    return;
+  }
+   
+  sub start {
+    my $class = shift;
+    
+    utl::cleanup(\$instance) if $ENV{'MOD_PERL'};
+
+    return $class->new();
+  }
+  
+  sub instance {
+    my $class = shift;
+    
+    return $instance;
+  }     
 
 }
 
